import { world, system } from "@minecraft/server";

const launcher = world.scoreboard.getObjective("launcher") ?? world.scoreboard.addObjective("launcher","LAUNCHER")
const pistol = world.scoreboard.getObjective("pistol") ?? world.scoreboard.addObjective("pistol","PISTOL")
const awm = world.scoreboard.getObjective("awm") ?? world.scoreboard.addObjective("awm","AWM")
const assault = world.scoreboard.getObjective("assault") ?? world.scoreboard.addObjective("assault","ASSAULT")
const musket = world.scoreboard.getObjective("musket") ?? world.scoreboard.addObjective("musket","MUSKET")

function toDegree(radian) {
  return (radian * 180) / Math.PI;
}

function viewDirectionToRotation(viewDirection) {
  const { x: vx, y: vy, z: vz } = viewDirection;
  const x = toDegree(Math.asin(-vy));
  const y = toDegree(Math.atan2(-vx, vz));
  return { x, y };
}

system.runInterval(() => {
    for (const player of world.getAllPlayers()) {
    const scoresLauncher = launcher.getScore(player);
    const scoresPistol = pistol.getScore(player);
    const scoresAwm = awm.getScore(player);
    const scoresAssault = assault.getScore(player);
    const scoresMusket = musket.getScore(player);
    const head = player.getHeadLocation(); 
    const view = player.getViewDirection();
    if (player.hasTag("assault") && scoresAssault >= 1 && scoresAssault <= 30){
        const spawnLocation = {
            x: head.x + view.x,
            y: head.y + view.y,
            z: head.z + view.z
        };
        assault.addScore(player, -1);
        const projectile = player.dimension.spawnEntity("takumi:takumi_assault_bullet",spawnLocation);
        const viewDirection = player.getViewDirection();
        const { x: rx, y: ry } = viewDirectionToRotation(viewDirection);
        projectile.applyImpulse({x:viewDirection.x*4,y:viewDirection.y*4,z:viewDirection.z*4});
        projectile.setRotation({ x: -rx, y: -ry });
    }
    player.runCommandAsync(`title @s[hasitem={location=slot.weapon.mainhand,item=takumi:takumi_launcher},tag=!reload] actionbar ${scoresLauncher}/3`)
    player.runCommandAsync(`title @s[hasitem={location=slot.weapon.mainhand,item=takumi:takumi_pistol},tag=!reload] actionbar ${scoresPistol}/15`)
    player.runCommandAsync(`title @s[hasitem={location=slot.weapon.mainhand,item=takumi:takumi_awm},tag=!reload] actionbar ${scoresAwm}/7`)
    player.runCommandAsync(`title @s[hasitem={location=slot.weapon.mainhand,item=takumi:takumi_assault},tag=!reload] actionbar ${scoresAssault}/30`)
    player.runCommandAsync(`title @s[hasitem={location=slot.weapon.mainhand,item=takumi:takumi_musket},tag=!reload] actionbar ${scoresMusket}/4`)
    player.runCommandAsync("title @s[tag=reload] actionbar Now reloading...")
}});

world.afterEvents.itemStopUse.subscribe(ev => { 
    const { itemStack, source } = ev;
    if (itemStack.typeId == "takumi:takumi_assault") {
        source.removeTag("assault");
    }
});

world.afterEvents.itemUse.subscribe(event => {
    const { itemStack, source } = event;
    const scoresLauncher = launcher.getScore(source);
    const scoresPistol = pistol.getScore(source);
    const scoresAwm = awm.getScore(source);
    const scoresAssault = assault.getScore(source);
    const scoresMusket = musket.getScore(source);
    const head = source.getHeadLocation(); 
    const view = source.getViewDirection();
    if (itemStack.typeId == "takumi:takumi_launcher" && scoresLauncher >= 1 && scoresLauncher <= 3){
        const spawnLocation = {
            x: head.x + view.x,
            y: head.y + view.y,
            z: head.z + view.z
        };
        launcher.addScore(source, -1);
        const projectile = source.dimension.spawnEntity("takumi:takumi_launcher_bullet",spawnLocation);
        const viewDirection = source.getViewDirection();
        const { x: rx, y: ry } = viewDirectionToRotation(viewDirection);
        projectile.applyImpulse({x:viewDirection.x*1,y:viewDirection.y*1,z:viewDirection.z*1});
        projectile.setRotation({ x: -rx, y: -ry });
    }
    if (!source.hasTag("reload") && itemStack.typeId == "takumi:takumi_launcher" && scoresLauncher <= 0){
    source.addTag("reload");
    source.playSound("random.fuse", {volume: 1.0, pitch: 1.0})
    system.runTimeout(() => {
        source.runCommandAsync("scoreboard players set @s[scores={launcher=0}] launcher 3")
        source.removeTag("reload");
    }, 56);
    }
    if (itemStack.typeId == "takumi:takumi_pistol" && scoresPistol >= 1 && scoresPistol <= 15){
        const spawnLocation = {
            x: head.x + view.x,
            y: head.y + view.y,
            z: head.z + view.z
        };
        pistol.addScore(source, -1);
        const projectile = source.dimension.spawnEntity("takumi:takumi_pistol_bullet",spawnLocation);
        const viewDirection = source.getViewDirection();
        const { x: rx, y: ry } = viewDirectionToRotation(viewDirection);
        projectile.applyImpulse({x:viewDirection.x*4,y:viewDirection.y*4,z:viewDirection.z*4});
        projectile.setRotation({ x: -rx, y: -ry });
    }
    if (!source.hasTag("reload") && itemStack.typeId == "takumi:takumi_pistol" && scoresPistol <= 0){
    source.addTag("reload");
    source.playSound("random.fuse", {volume: 1.0, pitch: 1.0})
    system.runTimeout(() => {
        source.runCommandAsync("scoreboard players set @s[scores={pistol=0}] pistol 15")
        source.removeTag("reload");
    }, 36);
    }
    if (itemStack.typeId == "takumi:takumi_awm" && scoresAwm >= 1 && scoresAwm <= 7){
        const spawnLocation = {
            x: head.x + view.x,
            y: head.y + view.y,
            z: head.z + view.z
        };
        awm.addScore(source, -1);
        const projectile = source.dimension.spawnEntity("takumi:takumi_awm_bullet",spawnLocation);
        const viewDirection = source.getViewDirection();
        const { x: rx, y: ry } = viewDirectionToRotation(viewDirection);
        projectile.applyImpulse({x:viewDirection.x*4,y:viewDirection.y*4,z:viewDirection.z*4});
        projectile.setRotation({ x: -rx, y: -ry });
    }
    if (!source.hasTag("reload") && itemStack.typeId == "takumi:takumi_awm" && scoresAwm <= 0){
    source.addTag("reload");
    source.playSound("random.fuse", {volume: 1.0, pitch: 1.0})
    system.runTimeout(() => {
        source.runCommandAsync("scoreboard players set @s[scores={awm=0}] awm 7")
        source.removeTag("reload");
    }, 60);
    }
    if (itemStack.typeId == "takumi:takumi_assault" && scoresAssault >= 1 && scoresAssault <= 30){
    source.addTag("assault");
    }
    if (!source.hasTag("reload") && itemStack.typeId == "takumi:takumi_assault" && scoresAssault <= 0){
    source.addTag("reload");
    source.playSound("random.fuse", {volume: 1.0, pitch: 1.0})
    system.runTimeout(() => {
        source.runCommandAsync("scoreboard players set @s[scores={assault=0}] assault 30")
        source.removeTag("reload");
    }, 46);
    }
    if (itemStack.typeId == "takumi:takumi_musket" && scoresMusket >= 1 && scoresMusket <= 4){
        const spawnLocation = {
            x: head.x + view.x,
            y: head.y + view.y,
            z: head.z + view.z
        };
        musket.addScore(source, -1);
        const projectile = source.dimension.spawnEntity("takumi:takumi_musket_bullet",spawnLocation);
        const viewDirection = source.getViewDirection();
        const { x: rx, y: ry } = viewDirectionToRotation(viewDirection);
        projectile.applyImpulse({x:viewDirection.x*4,y:viewDirection.y*4,z:viewDirection.z*4});
        projectile.setRotation({ x: -rx, y: -ry });
    }
    if (!source.hasTag("reload") && itemStack.typeId == "takumi:takumi_musket" && scoresMusket <= 0){
    source.addTag("reload");
    source.playSound("random.fuse", {volume: 1.0, pitch: 1.0})
    system.runTimeout(() => {
        source.runCommandAsync("scoreboard players set @s[scores={musket=0}] musket 4")
        source.removeTag("reload");
    }, 56);
    }
});